# SPDX-FileCopyrightText: 2024-present Mark Hall <mark.hall@open.ac.uk>
#
# SPDX-License-Identifier: MIT
"""The CLI entry-point."""

from tm352_app.cli import app

if __name__ == "__main__":
    app()
